function [f_mat_rho,sigma_mat,s_mat_rho,Flag] = Filter_SAR_GAS_rho_diag_t_tv_sigma_rho( k, T, intercepts, params_rho,params_sigma,nu, y_mat,W,ind_var_tar)

% DATE: 15-01-2025
% This function computes filteres volatilies and spatial dependencies
% according to the Dynamic diagonal SAR model with T distributed errors

% INPUT
% k             number of assets/countries 
% T:            number of obs
% intercepts    a k x 1 vector of intercepts for the SAR equation
% params_rho    a vector with parameters for the dynamic dependencies
% params_sigma  a vector with parameters for the dynamic volatilities
% nu            the DF parameter of the Mult T distribution
% y_mat:        a T by k matrix of data
% W:            a k by k matrix of spatial weights
% inv_var_tar:  0/1 funtion: 1 if you do variance targeting for the
% volatilities

% OUTPUT
% LLF:          the minus loglikelihood
% loglik_r2:    a T by 1 vector of loglikelihood values
% rho_mat:      a T by k matrix of spatial dependencies
% sigma_mat:    a T by k matrix of conditional variances
% A_mat         a k by k by T matrix of the matrix (I-R_t W)
% s_mat_rho     a T by k matrix of the score

% assign params and empty matrices
omega_rho = params_rho(1:k)';
alpha_rho =  params_rho(k+1)';
beta_rho  =  params_rho(end);
fbar_rho  = omega_rho./(1-beta_rho);

if ind_var_tar==1    
    fbar_sigma = log(var(y_mat));
    alpha_sigma = params_sigma(1);
    beta_sigma = params_sigma(2);
    omega_sigma = fbar_sigma*(1-beta_sigma);
else
    omega_sigma = params_sigma(1:k)';
    alpha_sigma = params_sigma(k+1);
    beta_sigma = params_sigma(end);
    fbar_sigma = omega_sigma./(1-beta_sigma);
end

I_k = eye(k);

f_mat_rho = zeros(T,k);
s_mat_rho = zeros(T,k);

s_mat_sigma = zeros(T,k);
f_mat_sigma = zeros(T,k);
sigma_mat = zeros(T,k);

f_mat_sigma(1,:) = fbar_sigma;
f_mat_rho(1,:) = fbar_rho;

Flag    = 0;
W_i     = W;

% loop over t and compute filtered volatilities and spatial dependencies
for i = 1:T
    
    h_t     = f_mat_rho(i,:);
    A_t     =  (I_k - diag(h_t)*W_i);
    
    [chol_A,ind_pd] = chol(A_t);
    
    if ind_pd>0 || rcond(A_t) < 1e-10
        Flag = 1;
        break
    end
    
    A_t_inv = A_t\I_k;
    
    sigma2_vec_t = exp(f_mat_sigma(i,:));
    sigma_mat(i,:) = sigma2_vec_t;
    
    Sigma_t = diag(sigma2_vec_t);
    Sigma_inv_t = Sigma_t\I_k;
    
    % build score
    y_t = y_mat(i,:)';
    B_t = A_t*y_t - intercepts;
    w_t = ((nu+k)/(nu-2))/ (1 + (1/(nu-2))*B_t' *Sigma_inv_t * B_t);
    
    W_Ainv = (W_i * A_t_inv);      
    dau_Bt_dau_vec_Rt = w_t * Sigma_inv_t * B_t * y_t'*W_i';
      
    s_t_0 = dau_Bt_dau_vec_Rt - W_Ainv;
    s_t_rho = det(A_t)* diag(s_t_0)';
    
    s_mat_rho(i,:) = s_t_rho;
    
    s_t_sigma_0 = 0.5* w_t * (Sigma_inv_t * (B_t * B_t')*Sigma_inv_t) - 0.5*Sigma_inv_t;
    s_t_sigma = diag(s_t_sigma_0)'.*sigma2_vec_t;
    s_mat_sigma(i,:) = s_t_sigma;
    
    % update tv parameters
    if i<T
        f_mat_rho(i+1,:) = omega_rho + alpha_rho* s_t_rho + beta_rho* f_mat_rho(i,:);
        f_mat_sigma(i+1,:) = omega_sigma + alpha_sigma* s_t_sigma + beta_sigma * f_mat_sigma(i,:);
    end
    
end

