function [LLF,loglik_r2,rho_mat,sigma_mat,A_mat,s_mat_rho] = LogLik_SAR_GAS_rho_diag_t_tv_sigma(T,params,y_mat,W,ind_var_tar);

% DATE: 15-01-2025
% This function computes the minus loglikelihood of the Dynamic diagonal
% SAR model with score driven dynamics for the volatility and spatial
% dependencies, assuming a multivariate T distribution for the data

% INPUT
% T:            number of obs
% params:       a vector with parameters
% y_mat:        a T by k matrix
% W:            a k by k matrix of spatial weights
% inv_var_tar:  0/1 funtion: 1 if you do variance targeting for the
% volatilities

% OUTPUT
% LLF:          the minus loglikelihood
% loglik_r2:    a T by 1 vector of loglikelihood values
% rho_mat:      a T by k matrix of spatial dependencies
% sigma_mat:    a T by k matrix of conditional variances
% A_mat         a k by k by T matrix of the matrix (I-R_t W)
% s_mat_rho     a T by k matrix of the score

k = size(y_mat,2);
I_k = eye(k);

params_rho = params(1:k+2);
params_sigma = params(k+3:end-1);
nu = params(end);

% For simplicity, intercepts are put to zero.
intercepts = zeros(k,1);

% filter tv parameters
[rho_mat,sigma_mat,s_mat_rho,Flag] = Filter_SAR_GAS_rho_diag_t_tv_sigma_rho(k, T,intercepts,params_rho,params_sigma,nu, y_mat,W,ind_var_tar);

A_mat = zeros(k,k,T);

% if the filtered paths are ok (Flag = 0), then fill the likelihood
if Flag ==0
    
    loglik_r2 = zeros(T,1);
    
    for j =1 :T
        
        Sigma_t         = diag(sigma_mat(j,:));
        inv_Sigma_t     = Sigma_t\I_k;
        logdet_Sigma_t  = log(det(Sigma_t));
        
        A_t = I_k - diag(rho_mat(j,:))*W;
        A_mat(:,:,j) = A_t;
        
        B_t = A_t *y_mat(j,:)';
        logdet_A_t = log(det(A_t));
        
        loglik_r2(j,1) = - 1/2 * logdet_Sigma_t  - 0.5*(nu + k) * log(1 + (1/(nu-2))*(B_t'*inv_Sigma_t*B_t)) + logdet_A_t;
        
    end
    
    loglik_r2 = loglik_r2  + gammaln(0.5*(nu + k)) - gammaln(nu/2) - 0.5*k*log((nu-2)*pi);
    LLF = -sum(loglik_r2);
    
else
    LLF = NaN;
    loglik_r2 = NaN;
end


if isnan(LLF)
    LLF = 1e12;
end

end
