function MAIN_SAR_models

% D'Innocenzo, Enzo, Andre Lucas, Anne Opschoor, and Xingmin Zhang (2024):
% "Heterogeneity and dynamics in network models", Journal of Applied Econometrics, 39(1), 150-173.
% email: a.opschoor@vu.nl

% Date: 15-01-2025
% this function simulate from a SAR model with score driven time-varying
% volatility and spatial dependencies.

% 15-1-2025: We thank Ye Shiqi for finding a typo in the code. This typo
% has been solved

%-------------------------------------------------------------------------
% set seed
rng(10);

%initialize parameters
T = 1500;
k = 7;
ind_var_tar = 0; 

alpha_GAS = 0.05;
beta_GAS = 0.98;
alpha_GAS_sigma = 0.04;
beta_GAS_sigma = 0.98;
nu = 5;

fbar_sigma = unifrnd(4,10,1,k);
fbar_log_sigma = log(fbar_sigma);
params_GAS_sigma = [fbar_log_sigma alpha_GAS_sigma beta_GAS_sigma]';

fbar_rho = linspace(-0.9,0.9,k);
params_GAS_rho = [fbar_rho alpha_GAS beta_GAS]';
theta_sim_real = [(1-beta_GAS)*fbar_rho alpha_GAS beta_GAS (1-beta_GAS_sigma)*fbar_log_sigma alpha_GAS_sigma beta_GAS_sigma nu]';

% load the W matrix (is static here) and row-normalize
load W_mat
W_mat_use = W_mat;
W_mat_RN = W_mat_use./(sum(W_mat_use,2)*ones(1,k));

% Simulate from the SAR model
[y_mat, rho_mat,s_mat_rho,f_mat_sigma,sigma2_mat,s_mat_sigma,Flag] = Simulate_SAR_GAS_rho_diag_tv_sigma(T,k,params_GAS_rho,params_GAS_sigma,nu,W_mat_RN);

% estimate back parameters
options = optimset('fmincon');
options = optimset(options, 'Display', 'off');
options = optimset(options, 'Diagnostics', 'off');
options = optimset(options, 'Algorithm', 'sqp');
options.MaxIter = 30000;
options.MaxFunEvals = 30000;
import LogLik.*

% Starting parameters plus left/right bounds

Para_start = [0.001*ones(k,1); 0.01; 0.97; 0.03*ones(k,1); 0.04; 0.95; 10];
Lb = [-3*ones(k,1); eps*ones(2,1); -0.50*ones(k,1); eps; 0.01; 2.1];
Rb = [50*ones(k,1); 0.8; 1-eps; 0.5*ones(k,1); 1; 1-eps; 50];

% define function and estimate parameters back by ML
f_SAR_GAS_rho_diag_t_tv_sigma_diag_alpha =  @(params)LogLik_SAR_GAS_rho_diag_t_tv_sigma(T,params,y_mat,W_mat_RN,ind_var_tar);
[theta_hat, fval_hat, Flag_hat] = fmincon(f_SAR_GAS_rho_diag_t_tv_sigma_diag_alpha,Para_start,[],[],[],[],Lb,Rb,[],options);
[~,~,rho_mat_hat,sigma2_mat_hat,A_mat_hat,s_mat_hat] =f_SAR_GAS_rho_diag_t_tv_sigma_diag_alpha(theta_hat);

% plot simulated and estimated volatitily/spillover paths
subplot(2,1,1)
plot([rho_mat rho_mat_hat])
title('Simulated and estimated spillover paths')
subplot(2,1,2)
plot([sigma2_mat sigma2_mat_hat])
title('Simulated and estimated volatility')


% print true and estimated parameters
theta_name = {'omega_1', 'omega_2', 'omega_3', 'omega_4', 'omega_5', 'omega_6', 'omega_7','a_SAR','b_SAR','omega_1_sig', 'omega_2_sig', 'omega_3_sig', 'omega_4_sig', 'omega_5_sig', 'omega_6_sig', 'omega_7_sig','a_vol','b_vol','nu'};
fprintf('Estimated parameters SAR model \n');
fprintf(' \n');
fprintf('  Para     True   Est \n');
nr_para = length(theta_hat);
for p = 1:nr_para
    fprintf(' %15s  %2.3f  %2.3f \n',theta_name{p},theta_sim_real(p),theta_hat(p));
end
fprintf(' \n');
fprintf('Maximized LogLikelihood: %8.1f \n', -fval_hat)






