function [y_mat, f_mat_rho,s_mat_rho,f_mat_sigma,sigma_mat,s_mat_sigma, Flag] = Simulate_SAR_GAS_rho_diag_tv_sigma(T,k,params_GAS_rho,params_GAS_sigma,nu,W)

% Date: 15-01-2025
% This function simulates from a Dynamic diagonal SAR model with time varying
% volatility and time varying diagonal Rho, assuming Multivariate T
% distributed errors

% y_t = A_t_inv eps_t with V(eps_t) = diag(sigma2_t)
% A_t is diagonal, follows a score driven process
% log sigma2_t also follows a score driven process

% Note 1: for the sake of simplicitly, I simulate from a scalar
% alfa (in both vol and R), of course a vector is also possible
% Note 2: I exclude constants and exogenous variables in the model for y_t
% Note 3: I use a FIXED W (a k by k matrix).

% INPUT
% T:                    number of obs
% k:                    number of assets
% params_GAS_rho:       parameters of the GAS spatial part
% params_GAS_sigma:     parameter of the volatility part
% nu:                   DF parameter of Mult T distribution
% W:                    Fixed Weight matrix

% OUTPUT
% y_mat:                A T x k matrix of simulated values of target variable
% f_mat_rho             A T x k matrix of filtered spatial dependencies
% s_mat_rho             A T x k matrix ofthe score values (the innovations)
%                       in the spatial dependency equation
% f_mat_sigma           A T x k matrix of the logarithm of variances
% sigma_mat             A T x k matrix of the variances
% s_mat_sigma           A T x k matrix of score values (the innovations) in
%                       the volatility equation
% FLAG                  1/0 varaible: 1 if (I- R W) is non positive
%                       definite



%--------------------------------------------------------------
% step 0; initialize vectors/matrices

y_mat = zeros(T,k);
f_mat_rho = zeros(T,k);
f_mat_sigma = zeros(T,k);

sigma_mat = zeros(T,k);
s_mat_rho = zeros(T,k);
s_mat_sigma = zeros(T,k);

fbar_rho = params_GAS_rho(1:k)';
alpha_GAS_rho = params_GAS_rho(k+1);
beta_GAS_rho = params_GAS_rho(k+2);
omega_rho = fbar_rho.*(1-beta_GAS_rho);
f_mat_rho(1,:) = fbar_rho;

fbar_sigma = params_GAS_sigma(1:k)';
alpha_GAS_sigma = params_GAS_sigma(end-1);
beta_GAS_sigma = params_GAS_sigma(end);

omega_sigma = fbar_sigma.*(1-beta_GAS_sigma);
f_mat_sigma(1,:) = fbar_sigma;
sigma_mat(1,:) =exp(fbar_sigma);

I_k = eye(k);
corr_factor_nu = sqrt((nu-2)/nu);
Flag = 0;

% step 1: loop over t, compute the score for the volatilities
% and the Rho mat.

for i = 1:T
    
    h_t     = f_mat_rho(i,:);
    A_t     =  (I_k - diag(h_t)*W);
    
    % check on PD ness
    [chol_A,ind_pd] = chol(A_t);
    
    if ind_pd>0 || rcond(A_t) < 1e-10
        Flag = 1;
        break
    end
    
    A_t_inv = A_t\I_k;
    sigma2_vec_t = exp(f_mat_sigma(i,:));
    sigma_mat(i,:) = sigma2_vec_t;
    
    % simulate the epsilon/y_t
    eps_vec = corr_factor_nu * mvtrnd(eye(k),nu,1) * diag(sqrt(sigma2_vec_t));
    y_t = A_t_inv * eps_vec';
    y_mat(i,:) = y_t;
    
    % compute the score for Rho and log sigma
    Sigma_t = diag(sigma2_vec_t);
    Sigma_inv_t = Sigma_t\I_k;
    
    B_t = A_t*y_t; %- Z_t\beta
    w_t = ((nu+k)/(nu-2))/ (1 + (1/(nu-2))*B_t' *Sigma_inv_t * B_t);
    
    W_Ainv = (W * A_t_inv);
    dau_Bt_dau_vec_Rt = w_t * Sigma_inv_t * B_t * y_t'*W';
    
    s_t_0 = dau_Bt_dau_vec_Rt - W_Ainv;
    s_t_rho = det(A_t)* diag(s_t_0)';
    s_mat_rho(i,:) = s_t_rho;
    
    s_t_sigma_0 = 0.5* w_t * (Sigma_inv_t * (B_t * B_t')*Sigma_inv_t) - 0.5*Sigma_inv_t;
    s_t_sigma = diag(s_t_sigma_0)'.*sigma2_vec_t;
    s_mat_sigma(i,:) = s_t_sigma;
    
    % given the score, build a new value for log sigma and R
    if i<T
        f_mat_rho(i+1,:) = omega_rho + alpha_GAS_rho * s_t_rho + beta_GAS_rho * f_mat_rho(i,:);
        f_mat_sigma(i+1,:) = omega_sigma + alpha_GAS_sigma * s_t_sigma + beta_GAS_sigma * f_mat_sigma(i,:);
    end
    
    
end


